@@ -21,13 +21,6 @@ def dump_decimal(self, value, write):
     self.dump_struct(value, write)
 
 
-def dump_bytes(self, value, write):
-    self.write = write
-    value = xmlrpc.client.Binary(value)
-    value.encode(self)
-    del self.write
-
-
 def dump_date(self, value, write):
     value = {'__class__': 'date',
         'year': value.year,
@@ -53,13 +46,35 @@ def dump_timedelta(self, value, write):
         }
     self.dump_struct(value, write)
 
+
+def dump_long(self, value, write):
+    try:
+        self.dump_long(value, write)
+    except OverflowError:
+        write('<value><biginteger>')
+        write(str(int(value)))
+        write('</biginteger></value>\n')
+
+
 xmlrpc.client.Marshaller.dispatch[Decimal] = dump_decimal
 xmlrpc.client.Marshaller.dispatch[datetime.date] = dump_date
 xmlrpc.client.Marshaller.dispatch[datetime.time] = dump_time
 xmlrpc.client.Marshaller.dispatch[datetime.timedelta] = dump_timedelta
-if bytes != str:
-    xmlrpc.client.Marshaller.dispatch[bytes] = dump_bytes
-xmlrpc.client.Marshaller.dispatch[bytearray] = dump_bytes
+xmlrpc.client.Marshaller.dispatch[int] = dump_long
+
+
+def dump_struct(self, value, write, escape=xmlrpc.client.escape):
+    converted_value = {}
+    for k, v in value.items():
+        if isinstance(k, int):
+            k = str(k)
+        elif isinstance(k, float):
+            k = repr(k)
+        converted_value[k] = v
+    return self.dump_struct(converted_value, write, escape=escape)
+
+
+xmlrpc.client.Marshaller.dispatch[dict] = dump_struct
 
 
 class XMLRPCDecoder(object):
@@ -76,6 +91,7 @@ class XMLRPCDecoder(object):
             return self.decoders[dct['__class__']](dct)
         return dct
 
+
 XMLRPCDecoder.register('date',
     lambda dct: datetime.date(dct['year'], dct['month'], dct['day']))
 XMLRPCDecoder.register('time',
@@ -97,6 +113,7 @@ def end_struct(self, data):
     self._stack[mark:] = [dct]
     self._value = 0
 
+
 xmlrpc.client.Unmarshaller.dispatch['struct'] = end_struct
 
 _CONFIG = threading.local()
@@ -215,6 +232,11 @@ class TrytondConfig(Config):
         super(TrytondConfig, self).__init__()
         if not database:
             database = os.environ.get('TRYTOND_DATABASE_URI')
+        elif (os.environ.get('TRYTOND_DATABASE_URI')
+                and not urllib.parse.urlparse(database).scheme):
+            url = urllib.parse.urlparse(os.environ['TRYTOND_DATABASE_URI'])
+            os.environ['TRYTOND_DATABASE_URI'] = urllib.parse.urlunparse(
+                url._replace(path=database))
         else:
             os.environ['TRYTOND_DATABASE_URI'] = database
         if not config_file:
@@ -308,7 +330,7 @@ class XmlrpcConfig(Config):
         super(XmlrpcConfig, self).__init__()
         self.url = url
         self.server = xmlrpc.client.ServerProxy(
-            url, allow_none=1, use_datetime=1, **kwargs)
+            url, allow_none=True, use_builtin_types=True, **kwargs)
         # TODO add user
         self.user = None
         self._context = self.server.model.res.user.get_preferences(True, {})
