@@ -153,6 +153,12 @@ class Eval(PYSON):
 
     @staticmethod
     def eval(dct, context):
+        if '.' in dct['v']:
+            base, name = dct['v'].split('.', 1)
+            return Eval.eval({
+                    'v': name,
+                    'd': dct['d'],
+                    }, context.get(base) or {})
         return context.get(dct['v'], dct['d'])
 
 
@@ -296,11 +302,15 @@ class Greater(PYSON):
         super(Greater, self).__init__()
         for i in (statement1, statement2):
             if isinstance(i, PYSON):
-                assert i.types().issubset({int, float, type(None)}), \
-                    'statement must be an integer or a float'
+                assert i.types().issubset({
+                        int, float, type(None),
+                        datetime.datetime, datetime.date}), \
+                    'statement must be an integer, float, date or datetime'
             else:
-                assert isinstance(i, (int, float, type(None))), \
-                    'statement must be an integer or a float'
+                assert isinstance(i, (
+                        int, float, type(None),
+                        datetime.datetime, datetime.date)), \
+                    'statement must be an integer, float, date or datetime'
         if isinstance(equal, PYSON):
             if equal.types() != {bool}:
                 equal = Bool(equal)
@@ -332,11 +342,19 @@ class Greater(PYSON):
                 dct[i] = 0.0
             if not isinstance(dct[i], (int, float)):
                 dct = dct.copy()
-                dct[i] = float(dct[i])
+                stmt = dct[i]
+                if isinstance(stmt, datetime.datetime):
+                    stmt = stmt.timestamp()
+                elif isinstance(stmt, datetime.date):
+                    time = datetime.time(0, 0)
+                    stmt = datetime.datetime.combine(stmt, time).timestamp()
+                dct[i] = float(stmt)
         return dct
 
     @staticmethod
     def eval(dct, context):
+        if dct['s1'] is None or dct['s2'] is None:
+            return False
         dct = Greater._convert(dct)
         if dct['e']:
             return dct['s1'] >= dct['s2']
@@ -353,6 +371,8 @@ class Less(Greater):
 
     @staticmethod
     def eval(dct, context):
+        if dct['s1'] is None or dct['s2'] is None:
+            return False
         dct = Less._convert(dct)
         if dct['e']:
             return dct['s1'] <= dct['s2']
@@ -495,7 +515,7 @@ class In(PYSON):
 class Date(PYSON):
 
     def __init__(self, year=None, month=None, day=None,
-            delta_years=0, delta_months=0, delta_days=0, **kwargs):
+            delta_years=0, delta_months=0, delta_days=0, start=None, **kwargs):
         year = kwargs.get('y', year)
         month = kwargs.get('M', month)
         day = kwargs.get('d', day)
@@ -516,11 +536,13 @@ class Date(PYSON):
         self._delta_years = delta_years
         self._delta_months = delta_months
         self._delta_days = delta_days
+        self._start = start
 
     @property
     def __repr_params__(self):
         return (self._year, self._month, self._day,
-            self._delta_years, self._delta_months, self._delta_days)
+            self._delta_years, self._delta_months, self._delta_days,
+            self._start)
 
     def pyson(self):
         return {
@@ -531,6 +553,7 @@ class Date(PYSON):
             'dy': self._delta_years,
             'dM': self._delta_months,
             'dd': self._delta_days,
+            'start': self._start,
             }
 
     def types(self):
@@ -538,7 +561,12 @@ class Date(PYSON):
 
     @staticmethod
     def eval(dct, context):
-        return datetime.date.today() + relativedelta(
+        today = dct.get('start')
+        if isinstance(today, datetime.datetime):
+            today = today.date()
+        if not isinstance(today, datetime.date):
+            today = datetime.date.today()
+        return today + relativedelta(
             year=dct['y'],
             month=dct['M'],
             day=dct['d'],
@@ -554,7 +582,7 @@ class DateTime(Date):
             hour=None, minute=None, second=None, microsecond=None,
             delta_years=0, delta_months=0, delta_days=0,
             delta_hours=0, delta_minutes=0, delta_seconds=0,
-            delta_microseconds=0, **kwargs):
+            delta_microseconds=0, start=None, **kwargs):
         hour = kwargs.get('h', hour)
         minute = kwargs.get('m', minute)
         second = kwargs.get('s', second)
@@ -565,7 +593,7 @@ class DateTime(Date):
         delta_microseconds = kwargs.get('dms', delta_microseconds)
         super(DateTime, self).__init__(year=year, month=month, day=day,
                 delta_years=delta_years, delta_months=delta_months,
-                delta_days=delta_days, **kwargs)
+                delta_days=delta_days, start=start, **kwargs)
         for i in (hour, minute, second, microsecond,
                 delta_hours, delta_minutes, delta_seconds, delta_microseconds):
             if isinstance(i, PYSON):
@@ -588,9 +616,10 @@ class DateTime(Date):
         date_params = super(DateTime, self).__repr_params__
         return (date_params[:3]
             + (self._hour, self._minute, self._second, self._microsecond)
-            + date_params[3:]
+            + date_params[3:-1]
             + (self._delta_hours, self._delta_minutes, self._delta_seconds,
-                self._delta_microseconds))
+                self._delta_microseconds)
+            + date_params[-1:])
 
     def pyson(self):
         res = super(DateTime, self).pyson()
@@ -610,7 +639,13 @@ class DateTime(Date):
 
     @staticmethod
     def eval(dct, context):
-        return datetime.datetime.now() + relativedelta(
+        now = dct.get('start')
+        if (isinstance(now, datetime.date)
+                and not isinstance(now, datetime.datetime)):
+            now = datetime.datetime.combine(now, datetime.time())
+        if not isinstance(now, datetime.datetime):
+            now = datetime.datetime.utcnow()
+        return now + relativedelta(
             year=dct['y'],
             month=dct['M'],
             day=dct['d'],
@@ -712,4 +747,6 @@ CONTEXT = {
     'DateTime': DateTime,
     'TimeDelta': TimeDelta,
     'Len': Len,
+    'true': True,
+    'false': False,
 }
